/* Copyright (c) 2006-2007 Christopher J. W. Lloyd

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

#import <Foundation/NSObject.h>
#import <Foundation/NSURLCache.h>

@protocol NSURLConnectionDelegate<NSObject>
@end

@protocol NSURLConnectionDataDelegate<NSURLConnectionDelegate>
@end

@class NSInputStream,NSOutputStream,NSRunLoop,NSURLRequest,NSData,NSURLResponse,NSError,NSMutableArray,NSURLAuthenticationChallenge,NSCachedURLResponse,NSURLProtocol,NSMutableData;

#import <Foundation/NSURLProtocol.h>

FOUNDATION_EXPORT_CLASS
@interface NSURLConnection : NSObject<NSURLProtocolClient> {
   id _request;
   id _protocol;
   id _delegate;
   id _response;
   unsigned _storagePolicy;
   BOOL _didRetain, _didRelease;
   id _mutableData;
}

+(BOOL)canHandleRequest:(NSURLRequest *)request;
+(NSData *)sendSynchronousRequest:(NSURLRequest *)request returningResponse:(NSURLResponse **)response error:(NSError **)error;

+(NSURLConnection *)connectionWithRequest:(NSURLRequest *)request delegate:delegate;
+(void)sendAsynchronousRequest:(NSURLRequest *)request queue:(NSOperationQueue *)queue completionHandler:(void (^)(NSURLResponse*, NSData*, NSError*))handler;
+(void)setDefaultAllowsInvalidSSLCertificate: (BOOL) allowInvalidCertificates;
-(BOOL)defaultAllowsInvalidSSLCertificate;

-initWithRequest:(NSURLRequest *)request delegate:delegate startImmediately:(BOOL)startLoading;
-initWithRequest:(NSURLRequest *)request delegate:delegate;

-(void)start;
-(void)cancel;

-(void)scheduleInRunLoop:(NSRunLoop *)runLoop forMode:(NSString *)mode;
-(void)unscheduleFromRunLoop:(NSRunLoop *)runLoop forMode:(NSString *)mode;

@end

@interface NSObject(NSURLConnectionDelegate)
-(void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error;
-(void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge;
-(void)connection:(NSURLConnection *)connection didCancelAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge;
-(void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data;
-(void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response;
-(NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)response;
-(NSURLRequest *)connection:(NSURLConnection *)connection willSendRequest:(NSURLRequest *)request redirectResponse:(NSURLResponse *)response;
-(void)connectionDidFinishLoading:(NSURLConnection *)connection;

@end
